/**
    @file
    @brief BLE module
    @copyright Copyright (c) 2012 Packet Power LLC. All Rights Reserved.
    @author Paul Bieganski

    This is the ble module which handles:
    - ble initialization routines

*/

#ifndef __P5_SVC_H__
#define __P5_SVC_H__

#include <stdint.h>
#include <stdbool.h>
#include "ble.h"

#define P5_BLE_UUID_BASE {0x23, 0xD1, 0xBC, 0xEA, 0x5F, 0x78, 0x23, 0x15, 0xDE, 0xEF, 0x12, 0x12, 0x00, 0x00, 0x00, 0x00}
#define P5_BLE_UUID_SERVICE 0xFECE
#define P5_BLE_UUID_LED_CHAR 0xFACE


typedef struct P5Svc
{
    uint16_t                    serviceHandle;
    ble_gatts_char_handles_t    ledCharHandles;
    uint8_t                     uuidType;
    uint8_t                     currentLedState;
    uint16_t                    connHandle;
    bool                        isNotifying;



} P5Svc_t;


uint32_t BleInit(P5Svc_t* p_p5Service);

void P5SvcOnBleEvt(P5Svc_t* const p_p5Service, 
                   ble_evt_t* const p_ble_evt);

uint32_t BleOnLedChange(P5Svc_t* p_p5Service, 
                        uint8_t ledState);

#endif // __P5_BLE_H__

/** @} */
